//
// Example code for 8-bit CFAF240400A0
//

#include <avr/io.h>
#include <util/delay.h>

#include "MMC_SD/MMC_SD.h"
#include "FAT/FAT.h"
#include "ili9327.h"

unsigned char * PATH = "\\batch";

extern uint16 SectorsPerClust;
extern uint16 FirstDataSector; 
extern uint8 FAT32_Enable;

struct FileInfoStruct FileInfo;

struct direntry PictureInfo;
uint16_t totalfiles = 0;

static uint16_t const WIDTH = 240;
static uint16_t const LENGTH = 400;

//==============================================================================
void init_cfaf40400a0(void)
{
	HIGH_BYTE = 0x00;
	LOW_BYTE = 0x00;

  //Deassert the read pin
	SET_RD;

 //**************Reset LCD Driver******
   SET_RESET;
   _delay_ms(1);
   CLR_RESET;
   _delay_ms(10);
   SET_RESET;
   _delay_ms(120);
//*****************************LCD SETING

  //Removing this because it's undefined
  //write_command(0xE9);
  //write_data(0x20);
  
  writeCommand(ILI9327_EXIT_SLEEP);
  _delay_ms(120);

  writeCommand(ILI9327_VCOM_CONTROL);
  writeData(0x00); //Default 0x00
  writeData(0x58); //Default 0x40
  writeData(0x15); //Default 0x0F

  writeCommand(ILI9327_POWER_SETTING);
  writeData(0x07);
  writeData(0x01);
  writeData(0x04);

  writeCommand(ILI9327_SET_ADDR_MODE);
  writeData(0x48);

  writeCommand(ili9327_SET_PIXEL_FORMAT);
  writeData(0x55);

  writeCommand(ILI9327_DISPLAY_TIMING);
  writeData(0x10);
  writeData(0x10);
  writeData(0x02);
  writeData(0x02);

  writeCommand(ILI9327_PANEL_DRIVE_SETTING); //Set Default Gamma
  writeData(0x00);
  writeData(0x35);
  writeData(0x00);
  writeData(0x00);
  writeData(0x01);
  writeData(0x02);

  writeCommand(ILI9327_FRAME_RATE_CONTROL); //Set frame rate
  writeData(0x02);

  writeCommand(ILI9327_POWER_SETTING_NORMAL); //power setting
  writeData(0x01);
  writeData(0x33);

  writeCommand(ILI9327_GAMMA_SETTING); //Set Gamma
  writeData(0x02);
  writeData(0x77);
  writeData(0x47);
  writeData(0x09);
  writeData(0x09);
  writeData(0x00);
  writeData(0x03);
  writeData(0x00);
  writeData(0x57);
  writeData(0x50);
  writeData(0x00);
  writeData(0x10);
  writeData(0x08);
  writeData(0x80);
  writeData(0x00);

  writeCommand(ILI9327_DISPLAY_ON);//DISPLAY ON
  _delay_ms(25); 

  writeCommand(ILI9327_3_GAMMA_FUNCTION_CONTROL); //Enable 3 Gamma
  writeData(0x00);

  displayHome();
}
//==============================================================================
void Picture_Slide_Show()
{
	uint8_t i;              //loop variable
	uint16_t j;             //loop variable
	uint32_t p;             //cluster
	uint16_t *buffer;				//buffer
	uint16_t pics = 1;
	uint16_t slide_show_flag = 1;
	uint8_t sector;
	uint32_t pixels;

	if (totalfiles == 0)
	return;

	buffer = malloc(512);
	if (buffer == 0)
	return;

next: 	//label for "goto"
	displayFillColor(BLACK);

	Search(PATH, &PictureInfo, &pics);                                         //find the file
	p = PictureInfo.deStartCluster + (((uint32)PictureInfo.deHighClust) << 16);//the first cluster of the file

	sector = 0;

	FAT_LoadPartCluster(p, sector, buffer);	//read a sector

	pixels = (uint32) 240 * 400; // total # of pixels to write

	displayHome();

	j = 0; // byte count
	
	while (pixels > 0)
	{
		writeColor(buffer[j++]);  // write 16 bits
    //write_data(buffer[j++]);  // write 16 bits
    pixels--;                 // which is one pixel
		if (j == 256)             // time for a new sector     
		{
			sector++;
			if (sector == SectorsPerClust)
			{
				p = FAT_NextCluster(p);				// read next cluster
				sector = 0;
			}

			FAT_LoadPartCluster(p, sector, buffer);	// read a sector
			j = 0;
		}  	
	}

	if (slide_show_flag)
	{
		for (i=0;i<30;i++)	// delay for a while
		_delay_ms(0xffff);

		pics++;					// increment picture number
		if (pics > totalfiles)	// if last
		pics = 1;			// wrap around

		goto next;				// go show next pic
	}
	free(buffer);
}
//==============================================================================
int main(void)
{
	PORTD = 0xF8;	    // pull-up on switches, all others off
	DDRD  = 0x0F;   	// VPP and led pins output

  //LOW_BYTE
	DDRA = 0xFF;    	// low data port, all outputs
  //HIGH_BYTE
	DDRE = 0xFF;    	// high data port, all outputs
	
  //Control pins LCD_CD, LCD_WR, LCD_RD, LCD_CS, LCD_RES
	PORTC = 0xFE;	    // all pins high
	DDRC  = 0xFE;	    // all outputs

	uint8 retry = 0;

	OSCCAL = 0x00;    //in order to operate some low speed card the initialization should run at lowest speed

	// Initialize the display
	init_cfaf40400a0();
  
  displayFillColor(RED);
  displayFillColor(GREEN);
  displayFillColor(BLUE);

  displayColorBars();
  
	MMC_SD_Init();    //SPI initialize

	sd_restart: 	//label for "goto"

	while (MMC_SD_Reset())  // Initialize SD Card
	{
		retry++;
		if (retry > 254)
		{
			while (1)
			{
				goto sd_restart;				// try again
			}
		}
	}

	OSCCAL = 0xff;      //run at high speed for normal operation for SD Card

	if (FAT_Init())   	//initialize file system - FAT16 and FAT32 are supported
	{
		while (1)
		{
			goto sd_restart;				// try again
		}
	}

	Search(PATH, &PictureInfo, &totalfiles);

	Picture_Slide_Show();

	while (1);
	return 0;

}
/*************************************************/
